@@ -1,5 +1,4 @@
-#include "torch_types.h"
-#include "utils.h"
+#include <torch.h>
 
 // [[Rcpp::export]]
 std::string cpp_layout_to_string(Rcpp::XPtr<XPtrTorchLayout> layout_ptr) {
@@ -7,11 +6,11 @@ std::string cpp_layout_to_string(Rcpp::XPtr<XPtrTorchLayout> layout_ptr) {
 }
 
 // [[Rcpp::export]]
-Rcpp::XPtr<XPtrTorchLayout> cpp_torch_strided () {
+Rcpp::XPtr<XPtrTorchLayout> cpp_torch_strided() {
   return make_xptr<XPtrTorchLayout>(lantern_Layout_strided());
 }
 
 // [[Rcpp::export]]
-Rcpp::XPtr<XPtrTorchLayout> cpp_torch_sparse () {
+Rcpp::XPtr<XPtrTorchLayout> cpp_torch_sparse() {
   return make_xptr<XPtrTorchLayout>(lantern_Layout_sparse());
 }
