@@ -1,14 +1,16 @@
-Sys.setenv(KMP_DUPLICATE_LIB_OK=TRUE)
-#torch_zeros(1, names="hello") # trigger warning about named tensors
+Sys.setenv(KMP_DUPLICATE_LIB_OK = TRUE)
+# torch_zeros(1, names="hello") # trigger warning about named tensors
 
 skip_if_not_test_examples <- function() {
-  if (Sys.getenv("TEST_EXAMPLES", unset = "0") != "1")
+  if (Sys.getenv("TEST_EXAMPLES", unset = "0") != "1") {
     skip("Not testing examples/readme. Set the env var TEST_EXAMPLES = 1.")
+  }
 }
 
 skip_if_cuda_not_available <- function() {
-  if (!cuda_is_available())
+  if (!cuda_is_available()) {
     skip("A GPU is not available for testing.")
+  }
 }
 
 expect_equal_to_tensor <- function(object, expected, ...) {
@@ -25,12 +27,12 @@ expect_no_error <- function(object, ...) {
 
 expect_tensor <- function(object) {
   expect_true(is_torch_tensor(object))
-  expect_no_error(as_array(object$to(device="cpu")))
+  expect_no_error(as_array(object$to(device = "cpu")))
 }
 
 expect_equal_to_r <- function(object, expected, ...) {
   expect_equal(as_array(object$cpu()), expected, ...)
-} 
+}
 
 expect_tensor_shape <- function(object, expected) {
   expect_tensor(object)
@@ -42,10 +44,8 @@ expect_undefined_tensor <- function(object) {
 }
 
 expect_identical_modules <- function(object, expected) {
-  
   expect_identical(
     attr(object, "module"),
-    attr(expected, "module")  
+    attr(expected, "module")
   )
-
-}
\ No newline at end of file
+}
